#ifndef IVS_DRAWER_H
#define IVS_DRAWER_H

#include "dhnetsdk.h"
#include <time.h>

#ifdef _WIN32
#define IVSDRAWER_API
#define CALLMETHOD __stdcall
#else
#define IVSDRAWER_API
#define CALLMETHOD
#endif

#ifdef _WIN32 
#include <windows.h>
#define IVS_BOOL        BOOL
#define IVS_WND_REF 	HWND
#define IVS_DC_REF		HDC
#define IVS_COLOR		COLORREF
#define IVS_RECT		RECT
typedef POINT			IVSPOINT;
#else
#define IVS_BOOL        int
#define IVS_WND_REF     void*
#define IVS_DC_REF      void*

typedef struct __IVS_MAC_WND
{
    int nWidth;
    int nHeight;
}IVS_WND;

typedef struct __IVS_COLOR
{
    float cgred;
    float cggreen;
    float cgblue;
}IVS_COLOR;

typedef struct __IVSPOINT
{
    float x;
    float y;
}IVSPOINT;

typedef struct __IVS_RECT
{
    float left;
    float bottom;
    float right;
    float top;
}IVS_RECT;
#endif

typedef enum
{
	SHOW_NONE = 0,
	SHOW_RULE = (1 << 0),
	SHOW_ALARM = (1 << 1),
	SHOW_TRACK = (1 << 2),
	SHOW_ALL = (SHOW_RULE | SHOW_ALARM | SHOW_TRACK),
}SHOW_TYPE;

typedef enum
{
	DRAW_JSON = 0,
	DRAW_TRACK,
	DRAW_ALARM,
	DRAW_RULE,
	DRAW_ALARMRULE,
	DRAW_ALARMEX,
	DRAW_TRACKEX,
	DRAW_MOVE_CHECK,
	DRAW_TEST = 9,
	DRAW_WEB_RULE = 11,
	DRAW_WEB_ALARM,
	DRAW_FLOW_INFO,
	DRAW_TRACKEX2,
	DRAW_WUXI235_TRACKEX2,
	DRAW_TRACKEXA1,
	DRAW_TRACKEX2_TYPE_HUMAN,
	DRAW_TRACKEX2_TYPE_VEHICLE,
	DRAW_TRACKEX2_TYPE_NONMOTOR,
	DRAW_TRACKEX2_TYPE_SHOPPRESENCE, 
	DRAW_TRACKEX2_TYPE_FLOWBUSINESS,
	DRAW_INTELFLOW,
	DRAW_SMARTMOTION,
	DRAW_END,
}DRAW_TYPE;

typedef enum
{
	IVS_LINGER = 0,				 
	IVS_TRACKTAIL,				// 켣β
	IVS_GET_OBJECT,
	IVS_DISABLE_VIDEO_TIME,		// ֹƵŨʱ䣬ĬϿ
	IVS_DISABLE_VIDEO_OBJECT,
	IVS_TRACK_OBJECT = 6,
	IVS_SET_PEN_HEIGHT,
	IVS_TRACK_OBJECT_EX = 8,
}IVS_CMD_TYPE;

typedef enum
{
    DRAW_PEN_DEFAULT = -1,
    DRAW_PEN_SOLID = 0,
    DRAW_PEN_DASH,                        
    DRAW_PEN_DOT,             
    DRAW_PEN_DASHDOT,         
    DRAW_PEN_DASHDOTDOT,       
    DRAW_PEN_NULL,             
    DRAW_PEN_INSIDEFRAME,     
    DRAW_PEN_USERSTYLE,        
    DRAW_PEN_ALTERNATE,        
}DRAW_PEN_STYLE;

#define IVS_MAX_ACTION_NUM		4
#define IVS_MAX_EVENT_NUM       64
#define IVS_MAX_POLYLINE_NUM	32
#define IVS_MAX_OBJECT_NUM		64
#define IVS_NAME_NUM		    128

typedef struct IVS_CONFIG_EVENT
{
	char					szEventName[IVS_NAME_NUM];			// ¼ƣ"¼б"
	char					szRuleName[IVS_NAME_NUM];			// ƣͬ
	DH_MSG_OBJECT			stuObject;
	char					Context[28];
	int						alarmAction;					    // NetSDKһ
	int						alarmType;
}IVS_CONFIG_EVENT_INFO;

typedef struct IVS_CONFIG_EVENTEX
{
	char					szEventName[IVS_NAME_NUM];			// ¼ƣ"¼б"
	char					szRuleName[IVS_NAME_NUM];			// ƣͬ
	char					Context[28];					
	int						alarmAction;					    // NetSDKһ
	int						alarmType;
	int						nObjectNum;
	DH_MSG_OBJECT			stuObject[IVS_MAX_OBJECT_NUM];		    // ⵽
	int						nContext;                           // ֶδãΪֶ
	void*					pContext;
}IVS_CONFIG_EVENT_INFOEX;

typedef struct 
{
	int						nEventsNum;							// Ƶ
	IVS_CONFIG_EVENT_INFO	stuEventInfo[IVS_MAX_EVENT_NUM];    // ¼Ϣ
}IVS_CFG_ANALYSEVENTS_INFO;

typedef struct 
{
	int						nEventsNum;							// Ƶ
	IVS_CONFIG_EVENT_INFOEX stuEventInfo[IVS_MAX_EVENT_NUM];    // ¼Ϣ
}IVS_CFG_ANALYSEVENTS_INFOEX;

typedef struct
{
	int						nX;                                 // 8192ϵ
	int						nY;		
}IVS_CFG_POLYLINE;

typedef struct  __IVS_WEB_RULE
{
	int						size;
	int						nRuleType;
	int						nRuleEnable;
	int						ndirect1;
	int						ndirect2;
	int						nPoint1;
	int						nPoint2;
	unsigned char			bActionType[4];
	DH_MSG_OBJECT			objectMsg;
	char					szRuleName[IVS_NAME_NUM];
	IVS_CFG_POLYLINE		stuDetectLine1[IVS_MAX_POLYLINE_NUM];	//Line
	IVS_CFG_POLYLINE		stuDetectLine2[IVS_MAX_POLYLINE_NUM];	//Region	
	int                     nRuleID;
	IVS_CFG_POLYLINE		stuDirectionLine[2];					//Direction
	char					szRevered[236];
}IVS_WEB_RULE;

typedef struct __IVS_WEB_RULE_ARRAY
{
	int					    size;
	int					    nCount;
	IVS_WEB_RULE*		    pRule;
}IVS_WEB_RULE_ARRAY;

typedef struct
{
	int					objectid;
	struct tm			startTime;
	struct tm			endTime;
	INT64				nBeginFileOffset;				// ʼʱļƫֽ(忪ʼʱƵ֡ԭʼƵļļʼƫ)
	INT64				nEndFileOffset;					// ʱļƫֽ(ʱƵ֡ԭʼƵļļʼƫ)
	int					classid;
}IVSOBJECT;

typedef struct
{
	IVS_BOOL            trackex2objtype;                // Ƿʾͣ0:1:ʾĬϲʾ
    IVS_BOOL            trackex2attribute88;            // Ƿʾ0x88԰0:1:ʾĬʾ
    IVS_BOOL            trackex2objid;                  // ǷʾID0:1:ʾĬϲʾ
    IVS_BOOL            trackex2humanage;	            // Ƿʾ䣬0:ʾΣ1:ʾ䣬Ĭʾ
}TrackEx2Configure;

typedef union
{
	IVSOBJECT			object;
	IVSPOINT			xPt;
	char				szReverd[128];
}ObjectContex;

typedef struct _DH_IVS_POINT
{
/*************************************************
** 켣Ӿε
** X,YXSize,YSizeӾΪ
** (X-XSize, Y-YSize, X+XSize, Y+YSize)
**************************************************/
	short 		x; 
	short 		y; 
	short		xSize;
	short		ySize;
}DH_IVS_POINT; 

typedef struct  _RuleColor
{
	int ruletype;
	IVS_COLOR crColor;		
}RuleColor;

#ifdef __cplusplus
extern "C" {
#endif

/**
 * IVSDraw(ֻһ)IvsDrawerjsonʱ
 */
IVSDRAWER_API int CALLMETHOD DRAW_Startup();

/**
 * ˳IVSDraw
 */
IVSDRAWER_API void CALLMETHOD DRAW_Cleanup();

/**
 * ͼ˿
 *
 * @param[in] nPort ͼ˿
 * @return BOOLɹTRUEʧܷFALSE
 */
IVSDRAWER_API BOOL CALLMETHOD DRAW_Open(int nPort);

/**
 * رͼ˿
 *
 * @param[in] nPort ͼ˿
 */
IVSDRAWER_API void CALLMETHOD DRAW_Close(int nPort);

/**
 * ͼ˿
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType  DRAW_TYPE壬DRAW_TRACK,DRAW_ALARM,DRAW_RULEЧ
 * @return BOOLɹTRUEʧܷFALSE
 */
IVSDRAWER_API BOOL CALLMETHOD DRAW_Reset(int nPort, int nType);

/**
 * jasonʽַݣԭʼδ
 *
 * @param[in] nPort ͼ˿
 * @param[in] pJsonData ڴַ
 * @param[in] nDataLen ݳ
 * @param[in] nPort ͼ˿
 * @param[in] nFrameSeq ֡
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_InputJsonData(int nPort, unsigned char* pJsonData, int nDataLen, int nFrameSeq);

/**
 * ùɫ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nRuleColor ƺ͹ɫ
 * @param[in] nRuleNum 
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_SetRuleColor(int nPort,RuleColor* nRuleColor, int nRuleNum);

/**
 * ݣjason
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType Ŀǰûõֶ
 * @param[in] pRuleData ڴַ
 * @param[in] nframe ֡
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_InputRuleData(int nPort, int nType, unsigned char* pRuleData, int nframe);

/**
 * 켣ݣӦܷ켣֡0xF1(ʾ֡)0x05(켣֡)
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType Ŀǰûõֶ
 * @param[in] pTrackData ڴַ
 * @param[in] nDataLen ݳ
 * @param[in] nFrameSeq ֡
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_InputTrackData(int nPort, int nType, unsigned char* pTrackData, int nDataLen,int nFrameSeq);

/**
 * 켣ݣܽṹϢ֡0xF1(ʾ֡)0x0E(ṹ)NVRŨϢ켣
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType nType=7ʾܽṹ֡nType=19ʾܿʾNVRŨϢ켣
 * @param[in] pTrackData ڴַ
 * @param[in] nDataLen ݳ
 * @param[in] nFrameSeq ֡
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_InputTrackDataEx2(int nPort, int nType, unsigned char* pTrackData, int nDataLen,int nFrameSeq);

/**
 * 뾯ݣjason
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType Ŀǰûõֶ
 * @param[in] pAlarmData ڴַ
 * @param[in] nDataLen ݳ
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_InputAlarmData(int nPort, int nType, unsigned char* pAlarmData, int nDataLen);

/**
 * 뾯ݣjasonDRAW_InputAlarmDataĲݽṹͬ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType Ŀǰûõֶ
 * @param[in] pAlarmData ڴַ
 * @param[in] nDataLen ݳ
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_InputAlarmDataEx(int nPort, int nType, unsigned char* pAlarmData, int nDataLen);

/**
 * 붯Ϣ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType Ŀǰûõֶ
 * @param[in] pData Ϣ
 * @param[in] nDataLen ݳ
 * @param[in] nFrameSeq ֡
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_InputMoveCheckData(int nPort, int nType, unsigned char* pData, int nDataLen, int nFrameSeq);

/**
 * ָλʾһַɶУ\nβ
 *
 * @param[in] nPort ͼ˿
 * @param[in] pTextData ݵַ
 * @param[in] nTextNumPos pTextDataݶӦĽṹ
 * @return intɹ0ɹ-1
 */
typedef struct
{
    const char* pText;  // ʾַ
	IVSPOINT hPos;      // ʾλ
	int nFontsize;      // С
	int nRed;           // ɫ
	int nGreen;         // ɫ
	int nBlue;          // ɫ
    int nAlignMode;     // 뷽ʽ0룬1ж룬2Ҷ
}TextData;
IVSDRAWER_API int CALLMETHOD DRAW_InputTextData(int nPort, TextData* pTextData, int nTextNumPos);

/**
 * ִͼ
 *
 * @param[in] nPort ͼ˿
 * @param[in] hDC ͼ豸
 * @param[in] hWnd ھ
 * @param[in] nFrameSeq ֡
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_Draw(int nPort, IVS_DC_REF hDC, IVS_WND_REF hWnd, int nFrameSeq);

/**
 * ʱ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType  DRAW_TYPE
 * @param[in] nLifeCount ʱ
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_SetLifeCount(int nPort, int nType, int nLifeCount);

/**
 * ʹ
 *
 * @param[in] nPort ͼ˿
 * @param[in] type  DRAW_TYPE
 * @param[in] bEnable ʹܱ
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_SetEnable(int nPort, int nType, IVS_BOOL bEnable);

/**
 * 
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType 
 * @param[in] nPenStyle ʸʽ(ʵor)DRAW_PEN_STYLE
 * @param[in] nWidth ʿ
 * @param[in] crColor ɫ
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_SetPen(int nPort, int nType, int nPenStyle, int nWidth, IVS_COLOR crColor);

/**
 * ִ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nCmdType IVS_CMD_TYPE
 * @param[in] pContext nCmdTypeΪIVS_TRACK_OBJECT/IVS_TRACK_OBJECT_EXIVS_GET_OBJECTָObjectContexṹ
 *                     ָint
 * @param[in] nContextSize pContextӦĽṹС
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_Ioctl(int nPort, int nCmdType, void* pContext, int nContextSize);

/**
 * ˢ֡
 *
 * @param[in] nPort ͼ˿
 * @param[in] nFrameSeq ֡
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_Refresh(int nPort, int nFrameSeq);

/**
 * ÿһΣһι(Ĭ255)ٵ0ʱɾ
 *
 * @param[in] nPort ͼ˿
 */
IVSDRAWER_API void CALLMETHOD DRAW_Idle(int nPort);

/**
 * ͼ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nReserved Ԥ
 */
IVSDRAWER_API void CALLMETHOD DRAW_Clean(int nPort, int nReserved);

/**
 * ù켣ʼʱ䣨ƵŨ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nTime ʱ
 * @param[in] nObjectId ID
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_SetTime(int nPort, int nTime, int nObjectId);

/**
 * ʾͣеrulealarmtrack
 *
 * @param[in] nPort     ͼ˿
 * @param[in] nShowType SHOW_TYPEʹ
 */
IVSDRAWER_API void CALLMETHOD DRAW_SetShowType(int nPort, int nShowType);

/**
 * trackص
 *
 * @param[in] nPort ͼ˿
 * @param[in] pCallBack ص
 * @param[in] pUserData ص
 */
typedef void (CALLMETHOD* OnDrawOneTrack)(int nClassID, int nObjID, DH_IVS_POINT* pPoints, int nPointNum, void* pUserData);
IVSDRAWER_API void CALLMETHOD DRAW_SetDrawOneTrackCallback(int nPort, OnDrawOneTrack pCallBack, void* pUserData);

/**
 * ַص
 *
 * @param[in] nPort ͼ˿
 * @param[in] pTranslateCBFunc ص
 * @param[in] pUserData ص
 */
typedef int (CALLMETHOD* fTranslateCallback)(unsigned char* strSrc,int nSrsLen, unsigned char* strDst, int* nDstLen, void* pUserData);
IVSDRAWER_API void CALLMETHOD DRAW_SetTranslateCallback(int nPort, fTranslateCallback pTranslateCBFunc, void* pUserData);

/**
 * ɴ˹trackᱻ
 *
 * @param[in] nPort ͼ˿
 * @param[in] pRuleName 
 * @param[in] bEnable Ƿ
 */
IVSDRAWER_API void CALLMETHOD DRAW_SetRuleTrackAlarm(int nPort, const char* pRuleName, bool bEnable);

/**
 * trackʾ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nType: 0ʾtrackĬϷʽ1ֻʾбtrack
 */
IVSDRAWER_API void CALLMETHOD DRAW_SetShowTrackType(int nPort, int nType);

/**
 * ֡
 *
 * @param[in] nPort ͼ˿
 * @param[in] nFrameSeq ֡
 */
IVSDRAWER_API void CALLMETHOD DRAW_SetFrameNum(int nPort, int nFrameSeq);

/**
 * {ID, ID}øɫ, ״
 *
 * @param[in] nPort ͼ˿
 * @param[in] nClassId ID
 * @param[in] nObjectId ID
 * @param[in] crColor ɫ
 * @param[in] bSpecialShape 1:״, 0:ĬϾ
 * @param[in] bSpecialSingle 1:ֻǰٿ״ٿصĬ״ 0״
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_SetTrackObjectColor(int nPort, int nClassId, int nObjectId, IVS_COLOR crColor, bool bSpecialShape, bool bSpecialSingle);

/**
 * bSpecialShape־óʶٿ״
 *
 * @param[in] nPort ͼ˿
 * @param[in] bSpecialShape 1:״, 0:ĬϾ
 * @return intɹ0ɹ-1
 */
IVSDRAWER_API int CALLMETHOD DRAW_SetTrackEX2Sharp(int nPort, bool bSpecialShape);

/**
 * trackex2configʾĳЩֶΣĬǲʾ
 *
 * @param[in] nPort ͼ˿
 * @param[in] trackex2config ǷʾĳЩֶ
 */
IVSDRAWER_API void CALLMETHOD DRAW_SetTrackEx2Config(int nPort, TrackEx2Configure trackex2config);

/**
 * bEnableƹǷʾĬʾ
 *
 * @param[in] nPort ͼ˿
 * @param[in] bEnable ֶ
 */
IVSDRAWER_API void CALLMETHOD DRAW_SetRuleNameConfig(int nPort, bool bEnable);

/**
 * NACLIOSƽ̨Уôڲӿ
 *
 * @param[in] nPort ͼ˿
 * @param[in] nX ½X
 * @param[in] nY ½Y
 * @param[in] nWidth ھο
 * @param[in] nHeight ھθ߶
 */
#if defined(NACL) || defined(IOS)
IVSDRAWER_API void CALLMETHOD DRAW_SetDrawSurfaceParam(int nPort, int nX, int nY, int nWidth, int nHeight);
#endif

/**
 * MACƽ̨нӿ
 * ⲿʾRetinaĻ
 * @param[in] nPort ͼ˿
 * @param[in] scale ʾ
 */
#if defined(MAC)
IVSDRAWER_API void CALLMETHOD DRAW_SetDisplayScale(int nPort, float scale);
#endif

/**
 * Чӿ
 */
IVSDRAWER_API int CALLMETHOD DRAW_GetLastError(int nPort);
IVSDRAWER_API int CALLMETHOD DRAW_InputJpegData(int nPort, int nType, unsigned char* pJpegData, int nDataLen, int nFrameSeq);


#ifdef __cplusplus
}
#endif

#endif
